import rt = require("raml-typesystem-interfaces");
import ti = rt.tsInterfaces;
import rjson = require("raml1-json-typings");
import raml = require("raml1-domain-model");
export declare abstract class Annotated {
    annotation(n: string): any;
    abstract annotations(): ti.IAnnotation[];
}
export declare abstract class Proxy<JSONType extends rjson.Annotable> extends Annotated implements ti.IAnnotatedElement {
    readonly json: JSONType;
    readonly parent: Proxy<any>;
    constructor(json: JSONType, parent: Proxy<any>);
    abstract name(): any;
    abstract kind(): any;
    owningFragment(): FragmentBase<any>;
    description(): any;
    annotationsMap(): any;
    private _annotations;
    annotations(): ti.IAnnotation[];
    value(): JSONType;
    entry(): JSONType;
}
export declare class Annotation extends Proxy<rjson.Annotation> implements raml.IAnnotation {
    name(): string;
    isInheritable(): boolean;
    facetName(): string;
    owner(): any;
    ownerFacet(): any;
    kind(): string;
    value(): any;
    definition(): raml.Type;
}
export declare abstract class FragmentBase<T> extends Proxy<T> {
    private tc;
    constructor(node: any, tc: ti.IParsedTypeCollection);
    getType(name: string): raml.Type;
    getTypeRegistry(): rt.tsInterfaces.ITypeRegistry;
    getAnnotationTypeRegistry(): rt.tsInterfaces.ITypeRegistry;
    types(): any[];
    annotationTypes(): any[];
    getAnnotationType(name: string): raml.Type;
}
export declare function mapArray<T extends Proxy<any>>(parent: Proxy<any>, property: string, clazz: {
    new (v: any, parent: Proxy<any>): T;
}): T[];
export declare function mapArrayMaps<T extends Proxy<any>>(parent: Proxy<any>, property: string, clazz: {
    new (v: any, parent: Proxy<any>): T;
}): T[];
export declare function mapMap<T extends Proxy<any>>(parent: Proxy<any>, property: string, clazz: {
    new (v: any, parent: Proxy<any>): T;
}): T[];
export declare abstract class LibraryBase<T> extends FragmentBase<T> {
    securitySchemes(): SecuritySchemeDefinition[];
}
export declare class SecuritySchemeRef extends Proxy<rjson.SecuritySchemeRef> implements raml.SecuredBy {
    name(): string;
    settings(): any;
    kind(): string;
    securitySchemes(): SecuritySchemeDefinition[];
}
export declare class Documentation extends Proxy<rjson.DocumentationItem> {
    name(): string;
    title(): string;
    content(): string;
    kind(): string;
}
export declare class Api extends LibraryBase<rjson.Api> implements raml.Api {
    baseUri(): string;
    protocols(): string[];
    mediaType(): string[];
    baseUriParameters(): Parameter[];
    documentation(): Documentation[];
    securedBy(): SecuritySchemeRef[];
    version(): string;
    kind(): string;
    name(): string;
    title(): string;
    resources(): Resource[];
    allResources(): raml.Resource[];
    allMethods(): raml.Method[];
}
export declare class Response extends Proxy<rjson.Response> implements raml.Response {
    name(): string;
    code(): string;
    headers(): Parameter[];
    bodies(): Body[];
    method(): Method;
    kind(): string;
}
export declare class Body extends Annotated implements raml.Body {
    private mime;
    private p;
    constructor(mime: string, p: ti.IParsedType);
    mimeType(): string;
    type(): rt.tsInterfaces.IParsedType;
    annotations(): rt.tsInterfaces.IAnnotation[];
}
export declare class Parameter extends Annotated implements raml.Parameter {
    private mime;
    private p;
    private req;
    private loc;
    constructor(mime: string, p: ti.IParsedType, req: boolean, loc: string);
    required(): boolean;
    location(): string;
    name(): string;
    type(): rt.tsInterfaces.IParsedType;
    annotations(): rt.tsInterfaces.IAnnotation[];
}
export declare class Method extends Proxy<rjson.Method> implements raml.Method {
    name(): string;
    protocols(): string[];
    securedBy(): raml.SecuredBy[];
    kind(): string;
    displayName(): string;
    method(): string;
    parameters(): raml.Parameter[];
    bodies(): Body[];
    responses(): Response[];
    resource(): Resource;
}
export declare class Library extends LibraryBase<rjson.Library> {
    usage(): string;
    kind(): string;
    name(): string;
}
export declare class SecuritySchemeDefinition extends Proxy<rjson.SecuritySchemeDefinition> implements raml.SecuritySchemeDefinition {
    settings(): rjson.Map<any>;
    type(): string;
    name(): string;
    kind(): string;
}
export declare class Resource extends Proxy<rjson.Resource> implements raml.Resource {
    securedBy(): raml.SecuredBy[];
    relativeUrl(): string;
    fullRelativeUrl(): string;
    absoluteUrl(): string;
    parentResource(): raml.Resource;
    uriParameters(): Parameter[];
    allUriParameters(): raml.Parameter[];
    resources(): raml.Resource[];
    owningApi(): raml.Api;
    methods(): Method[];
    name(): string;
    kind(): string;
}
export declare function loadApi(api: rjson.Api): raml.Api;
export declare function loadLibrary(api: rjson.Library): raml.Library;
