"use strict";
var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
};
var rt = require("raml-typesystem-interfaces");
var ti = rt.tsInterfaces;
var ts = require("raml-typesystem-light");
function normalizeTypes(target, base, types, resultName) {
    if (resultName === void 0) { resultName = types; }
    if (base[types]) {
        base[types].forEach(function (t) {
            var nm = Object.keys(t)[0];
            var type = t[Object.keys(t)[0]];
            if (typeof t == "string") {
                target[resultName][nm] = t;
            }
            else if (Array.isArray(t)) {
                target[resultName][nm] = { type: t };
            }
            else {
                target[resultName][nm] = normalizeType(type);
            }
        });
    }
}
function normalizeType(i) {
    if (typeof i == "string") {
        return { type: i };
    }
    if (typeof i == "object") {
        if (Array.isArray(i)) {
            return { type: i };
        }
        var t_1 = i;
        var res = JSON.parse(JSON.stringify(t_1));
        delete res.name;
        delete res.typePropertyKind;
        delete res.annotations;
        delete res.structuredExample;
        if (t_1.structuredExample) {
            var e = {
                value: t_1.structuredExample.structuredValue
            };
            if (t_1.structuredExample.annotations) {
                Object.keys(t_1.structuredExample.annotations).forEach(function (x) {
                    e['(' + x + ')'] = t_1.structuredExample.annotations[x].structuredValue;
                });
            }
            res.example = e;
        }
        if (t_1.examples) {
            var examples = [];
            t_1.examples.forEach(function (e) {
                var ex = { value: e.structuredValue };
                if (e.annotations) {
                    Object.keys(e.annotations).forEach(function (x) {
                        ex['(' + x + ')'] = e.annotations[x].structuredValue;
                    });
                }
                examples.push(ex);
            });
            res.examples = examples;
        }
        if (t_1.annotations) {
            Object.keys(t_1.annotations).forEach(function (x) {
                res['(' + x + ")"] = t_1.annotations[x].structuredValue;
            });
        }
        if (t_1.properties) {
            Object.keys(t_1.properties).forEach(function (x) {
                res.properties[x] = normalizeType(t_1.properties[x]);
            });
        }
        if (t_1.facets) {
            Object.keys(t_1.facets).forEach(function (x) {
                res.facets[x] = normalizeType(t_1.facets[x]);
            });
        }
    }
    return res;
}
function normalize(api) {
    var rs = { types: {}, annotationTypes: {} };
    normalizeTypes(rs, api, "types");
    normalizeTypes(rs, api, "schemas");
    normalizeTypes(rs, api, "annotationTypes");
    return rs;
}
var Annotated = (function () {
    function Annotated() {
    }
    Annotated.prototype.annotation = function (n) {
        var res = null;
        this.annotations().forEach(function (x) {
            if (x.name() == n || x.name().endsWith("." + n)) {
                res = x;
            }
        });
        if (res) {
            return res.value();
        }
        return null;
    };
    return Annotated;
}());
exports.Annotated = Annotated;
var Proxy = (function (_super) {
    __extends(Proxy, _super);
    function Proxy(json, parent) {
        _super.call(this);
        this.json = json;
        this.parent = parent;
    }
    Proxy.prototype.owningFragment = function () {
        if (this.parent) {
            return this.parent.owningFragment();
        }
        if (this instanceof FragmentBase) {
            return this;
        }
        return null;
    };
    Proxy.prototype.description = function () {
        return this.json.description || this.json.usage;
    };
    Proxy.prototype.annotationsMap = function () {
        return null;
    };
    Proxy.prototype.annotations = function () {
        var _this = this;
        if (this._annotations) {
            return this._annotations;
        }
        var result = [];
        if (this.json.annotations) {
            Object.keys(this.json.annotations).forEach(function (x) {
                var v = _this.json.annotations[x];
                result.push(new Annotation(v, _this));
            });
        }
        this._annotations = result;
        return this._annotations;
    };
    Proxy.prototype.value = function () {
        return this.json;
    };
    Proxy.prototype.entry = function () {
        return this.json;
    };
    return Proxy;
}(Annotated));
exports.Proxy = Proxy;
var Annotation = (function (_super) {
    __extends(Annotation, _super);
    function Annotation() {
        _super.apply(this, arguments);
    }
    Annotation.prototype.name = function () {
        return this.json.name;
    };
    Annotation.prototype.isInheritable = function () {
        return false;
    };
    Annotation.prototype.facetName = function () {
        return this.json.name;
    };
    Annotation.prototype.owner = function () {
        return null;
    };
    Annotation.prototype.ownerFacet = function () {
        return null;
    };
    Annotation.prototype.kind = function () {
        return "Annotation";
    };
    Annotation.prototype.value = function () {
        return this.json.structuredValue;
    };
    Annotation.prototype.definition = function () {
        var library = this.owningFragment();
        var tp = library.getAnnotationType(this.json.name);
        if (tp) {
            return tp;
        }
        return null;
    };
    return Annotation;
}(Proxy));
exports.Annotation = Annotation;
function unique(v) {
    return Array.from(new Set(v));
}
var FragmentBase = (function (_super) {
    __extends(FragmentBase, _super);
    function FragmentBase(node, tc) {
        _super.call(this, node, null);
        this.tc = tc;
    }
    FragmentBase.prototype.getType = function (name) {
        return this.tc.getType(name);
    };
    FragmentBase.prototype.getTypeRegistry = function () {
        return this.tc.getTypeRegistry();
    };
    FragmentBase.prototype.getAnnotationTypeRegistry = function () {
        return this.tc.getAnnotationTypeRegistry();
    };
    FragmentBase.prototype.types = function () {
        return unique(this.tc.types());
    };
    FragmentBase.prototype.annotationTypes = function () {
        return unique(this.tc.annotationTypes());
    };
    FragmentBase.prototype.getAnnotationType = function (name) {
        return this.tc.getAnnotationType(name);
    };
    return FragmentBase;
}(Proxy));
exports.FragmentBase = FragmentBase;
function mapArray(parent, property, clazz) {
    var obj = parent.json[property];
    if (!obj) {
        obj = [];
    }
    return obj.map(function (x) { return x == null ? x : new clazz(x, parent); });
}
exports.mapArray = mapArray;
function mapArrayMaps(parent, property, clazz) {
    var obj = parent.json[property];
    if (!obj) {
        obj = [];
    }
    return obj.map(function (x) { return new clazz(x[Object.keys(x)[0]], parent); });
}
exports.mapArrayMaps = mapArrayMaps;
function mapMap(parent, property, clazz) {
    var obj = parent.json[property];
    if (!obj) {
        obj = {};
    }
    var res = [];
    Object.keys(obj).forEach(function (x) {
        res.push(new clazz(obj[x], parent));
    });
    return res;
}
exports.mapMap = mapMap;
function gatherResources(r, res) {
    var resources = r.resources();
    resources.forEach(function (x) {
        res.push(x);
        gatherResources(x, res);
    });
}
var LibraryBase = (function (_super) {
    __extends(LibraryBase, _super);
    function LibraryBase() {
        _super.apply(this, arguments);
    }
    LibraryBase.prototype.securitySchemes = function () {
        return mapArrayMaps(this, "securitySchemes", SecuritySchemeDefinition);
    };
    return LibraryBase;
}(FragmentBase));
exports.LibraryBase = LibraryBase;
var SecuritySchemeRef = (function (_super) {
    __extends(SecuritySchemeRef, _super);
    function SecuritySchemeRef() {
        _super.apply(this, arguments);
    }
    SecuritySchemeRef.prototype.name = function () {
        if (typeof this.json == "string") {
            return this.json;
        }
        return Object.keys(this.json)[0];
    };
    SecuritySchemeRef.prototype.settings = function () {
        if (typeof this.json == "string") {
            return {};
        }
        return this.json[Object.keys(this.json)[0]];
    };
    SecuritySchemeRef.prototype.kind = function () {
        return "SecuritySchemeRef";
    };
    SecuritySchemeRef.prototype.securitySchemes = function () {
        return mapArrayMaps(this, "securitySchemes", SecuritySchemeDefinition);
    };
    return SecuritySchemeRef;
}(Proxy));
exports.SecuritySchemeRef = SecuritySchemeRef;
var Documentation = (function (_super) {
    __extends(Documentation, _super);
    function Documentation() {
        _super.apply(this, arguments);
    }
    Documentation.prototype.name = function () {
        return this.json.title;
    };
    Documentation.prototype.title = function () {
        return this.json.title;
    };
    Documentation.prototype.content = function () {
        return this.json.content;
    };
    Documentation.prototype.kind = function () {
        return "Documentation";
    };
    return Documentation;
}(Proxy));
exports.Documentation = Documentation;
var Api = (function (_super) {
    __extends(Api, _super);
    function Api() {
        _super.apply(this, arguments);
    }
    Api.prototype.baseUri = function () {
        return this.json.baseUri;
    };
    Api.prototype.protocols = function () {
        return this.json.protocols;
    };
    Api.prototype.mediaType = function () {
        var s = this.json.mediaType;
        if (typeof s == "string") {
            return [s];
        }
        else {
            return s;
        }
    };
    Api.prototype.baseUriParameters = function () {
        if (this.json.baseUriParameters) {
            return params(this.json.baseUriParameters, this, "baseUri");
        }
        return [];
    };
    Api.prototype.documentation = function () {
        return mapArray(this, "documentation", Documentation);
    };
    Api.prototype.securedBy = function () {
        return mapArray(this, "securedBy", SecuritySchemeRef);
    };
    Api.prototype.version = function () {
        return this.json.version;
    };
    Api.prototype.kind = function () {
        return "Api";
    };
    Api.prototype.name = function () {
        return this.title();
    };
    Api.prototype.title = function () {
        return this.json.title;
    };
    Api.prototype.resources = function () {
        return mapArray(this, "resources", Resource);
    };
    Api.prototype.allResources = function () {
        var res = [];
        gatherResources(this, res);
        return res;
    };
    Api.prototype.allMethods = function () {
        var meth = [];
        this.allResources().forEach(function (x) {
            meth = meth.concat(x.methods());
        });
        return meth;
    };
    return Api;
}(LibraryBase));
exports.Api = Api;
function bodies(t) {
    if (!t.json.body) {
        return [];
    }
    var result = [];
    Object.keys(t.json.body).forEach(function (x) {
        var td = normalizeType(t.json.body[x]);
        var parsedType = ts.parseJsonTypeWithCollection("", td, t.owningFragment(), true);
        result.push(new Body(x, parsedType));
    });
    return result;
}
function isRequired(parsedType) {
    return parsedType.allFacets().some(function (x) { return x.kind() == ti.MetaInformationKind.Required && x.value(); });
}
function params(v, parent, location) {
    var result = [];
    if (v) {
        Object.keys(v).forEach(function (x) {
            var td = normalizeType(v[x]);
            var parsedType = ts.parseJsonTypeWithCollection("", td, parent.owningFragment(), false);
            result.push(new Parameter(x, parsedType, isRequired(parsedType), location));
        });
    }
    return result;
}
function paramsFromArray(v, parent, location) {
    var result = [];
    if (v) {
        v.forEach(function (x) {
            var td = normalizeType(x);
            var parsedType = ts.parseJsonTypeWithCollection("", td, parent.owningFragment(), false);
            result.push(new Parameter(x.name, parsedType, isRequired(parsedType), location));
        });
    }
    return result;
}
var Response = (function (_super) {
    __extends(Response, _super);
    function Response() {
        _super.apply(this, arguments);
    }
    Response.prototype.name = function () {
        return this.json.code;
    };
    Response.prototype.code = function () {
        return this.json.code;
    };
    Response.prototype.headers = function () {
        return paramsFromArray(this.json.headers, this, "responseHeaders");
    };
    Response.prototype.bodies = function () {
        return bodies(this);
    };
    Response.prototype.method = function () {
        return this.parent;
    };
    Response.prototype.kind = function () {
        return "Response";
    };
    return Response;
}(Proxy));
exports.Response = Response;
var Body = (function (_super) {
    __extends(Body, _super);
    function Body(mime, p) {
        _super.call(this);
        this.mime = mime;
        this.p = p;
    }
    Body.prototype.mimeType = function () {
        return this.mime;
    };
    Body.prototype.type = function () {
        return this.p;
    };
    Body.prototype.annotations = function () {
        return this.p.annotations();
    };
    return Body;
}(Annotated));
exports.Body = Body;
var Parameter = (function (_super) {
    __extends(Parameter, _super);
    function Parameter(mime, p, req, loc) {
        _super.call(this);
        this.mime = mime;
        this.p = p;
        this.req = req;
        this.loc = loc;
    }
    Parameter.prototype.required = function () {
        return this.req;
    };
    Parameter.prototype.location = function () {
        return this.loc;
    };
    Parameter.prototype.name = function () {
        return this.mime;
    };
    Parameter.prototype.type = function () {
        return this.p;
    };
    Parameter.prototype.annotations = function () {
        return this.p.annotations();
    };
    return Parameter;
}(Annotated));
exports.Parameter = Parameter;
var Method = (function (_super) {
    __extends(Method, _super);
    function Method() {
        _super.apply(this, arguments);
    }
    Method.prototype.name = function () {
        return this.json.method;
    };
    Method.prototype.protocols = function () {
        if (this.json.protocols) {
            return this.json.protocols;
        }
        return [];
    };
    Method.prototype.securedBy = function () {
        if (!this.json.securedBy && this.resource()) {
            return this.resource().securedBy();
        }
        return mapArray(this, "securedBy", SecuritySchemeRef);
    };
    Method.prototype.kind = function () {
        return "Method";
    };
    Method.prototype.displayName = function () {
        return this.json.displayName;
    };
    Method.prototype.method = function () {
        return this.json.method;
    };
    Method.prototype.parameters = function () {
        var initial = params(this.json.queryParameters, this, "query").concat(params(this.json.headers, this, "headers"));
        initial = this.resource().allUriParameters().concat(initial);
        if (this.json.queryString) {
            var td = normalizeType(this.json.queryString);
            var parsedType = ts.parseJsonTypeWithCollection("", td, parent, false);
            initial.push(new Parameter("queryString", parsedType, isRequired(parsedType), "queryString"));
        }
        return initial;
    };
    Method.prototype.bodies = function () {
        return bodies(this);
    };
    Method.prototype.responses = function () {
        return mapMap(this, "responses", Response);
    };
    Method.prototype.resource = function () {
        if (this.parent instanceof Resource) {
            return this.parent;
        }
        return null;
    };
    return Method;
}(Proxy));
exports.Method = Method;
var Library = (function (_super) {
    __extends(Library, _super);
    function Library() {
        _super.apply(this, arguments);
    }
    Library.prototype.usage = function () {
        return this.json.usage;
    };
    Library.prototype.kind = function () {
        return "Library";
    };
    Library.prototype.name = function () {
        return "";
    };
    return Library;
}(LibraryBase));
exports.Library = Library;
var SecuritySchemeDefinition = (function (_super) {
    __extends(SecuritySchemeDefinition, _super);
    function SecuritySchemeDefinition() {
        _super.apply(this, arguments);
    }
    SecuritySchemeDefinition.prototype.settings = function () {
        return this.json.settings;
    };
    SecuritySchemeDefinition.prototype.type = function () {
        return this.json.type;
    };
    SecuritySchemeDefinition.prototype.name = function () {
        return this.json.name;
    };
    SecuritySchemeDefinition.prototype.kind = function () {
        return "SecuritySchemeDefinition";
    };
    return SecuritySchemeDefinition;
}(Proxy));
exports.SecuritySchemeDefinition = SecuritySchemeDefinition;
var Resource = (function (_super) {
    __extends(Resource, _super);
    function Resource() {
        _super.apply(this, arguments);
    }
    Resource.prototype.securedBy = function () {
        if (!this.json.securedBy && this.owningApi()) {
            return this.owningApi().securedBy();
        }
        return mapArray(this, "securedBy", SecuritySchemeRef);
    };
    Resource.prototype.relativeUrl = function () {
        return this.json.relativeUri;
    };
    Resource.prototype.fullRelativeUrl = function () {
        var m = this.json.relativeUri;
        var pr = this.parentResource();
        if (pr) {
            m = pr.fullRelativeUrl() + m;
        }
        return m;
    };
    Resource.prototype.absoluteUrl = function () {
        return this.json.absoluteUri;
    };
    Resource.prototype.parentResource = function () {
        if (this.parent instanceof Resource) {
            return this.parent;
        }
        return null;
    };
    Resource.prototype.uriParameters = function () {
        return params(this.json.uriParameters, this, "uri");
    };
    Resource.prototype.allUriParameters = function () {
        var p = this.parentResource();
        if (p) {
            return p.allUriParameters().concat(this.uriParameters());
        }
        return this.uriParameters();
    };
    Resource.prototype.resources = function () {
        return mapArray(this, "resources", Resource);
    };
    Resource.prototype.owningApi = function () {
        return this.owningFragment();
    };
    Resource.prototype.methods = function () {
        return mapArray(this, "methods", Method);
    };
    Resource.prototype.name = function () {
        return this.json.displayName;
    };
    Resource.prototype.kind = function () {
        return "Resource";
    };
    return Resource;
}(Proxy));
exports.Resource = Resource;
function loadApi(api) {
    var nm = normalize(api);
    var collection = ts.parseJSONTypeCollection(nm);
    return new Api(api, collection);
}
exports.loadApi = loadApi;
function loadLibrary(api) {
    var nm = normalize(api);
    var collection = ts.parseJSONTypeCollection(nm);
    return new Library(api, collection);
}
exports.loadLibrary = loadLibrary;
//# sourceMappingURL=main.js.map